"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const getSymbolTableValues = (table) => {
    const values = [];
    const iter = table.values();
    for (let next = iter.next(); !next.done; next = iter.next()) {
        values.push(next.value);
    }
    return values;
};
const getSimpleTypeName = (typescript, checker, type) => checker.typeToString(type, undefined, typescript.TypeFormatFlags.InTypeAlias |
    typescript.TypeFormatFlags.NoTruncation);
const getUnionTypeName = (typescript, checker, type) => type.types
    .map(type => getTypeName(typescript, checker, type))
    .join(type.isUnion() ? " | " : " & ");
const getTypeArgumentNames = (typescript, checker, type) => {
    if (type.flags & typescript.TypeFlags.Object) {
        const otype = type;
        if (otype.objectFlags & typescript.ObjectFlags.Reference) {
            const rtype = type;
            if (rtype.typeArguments) {
                return rtype.typeArguments.map(arg => getTypeName(typescript, checker, arg));
            }
        }
    }
    return [];
};
const getTypeName = (typescript, checker, type) => {
    if (type.symbol &&
        type.symbol.flags & typescript.SymbolFlags.Interface &&
        type.symbol.escapedName === "Array") {
        const typeArgumentNames = getTypeArgumentNames(typescript, checker, type);
        return `Array<${typeArgumentNames.join(", ")}>`;
    }
    else if (type.symbol &&
        (type.symbol.flags & typescript.SymbolFlags.Interface ||
            type.symbol.flags & typescript.SymbolFlags.TypeLiteral)) {
        const typeArgumentNames = getTypeArgumentNames(typescript, checker, type);
        let name = "{ ";
        if (type.symbol.members) {
            const members = getSymbolTableValues(type.symbol.members);
            const typeParamTypes = members
                .filter(member => member.flags & typescript.SymbolFlags.TypeParameter)
                .map((member, i) => [
                member.escapedName,
                typeArgumentNames[i]
            ])
                .reduce((acc, [name, type]) => ({ ...acc, [name]: type }), {});
            for (const member of members.filter(member => member.flags & typescript.SymbolFlags.Property)) {
                let memberType = getNodeTypeName(typescript, checker, member.valueDeclaration);
                if (typeParamTypes[memberType]) {
                    memberType = typeParamTypes[memberType];
                }
                name += `${member.escapedName}: ${memberType}; `;
            }
        }
        name += "}";
        return name;
    }
    else if (type.isUnionOrIntersection()) {
        return getUnionTypeName(typescript, checker, type);
    }
    else {
        return getSimpleTypeName(typescript, checker, type);
    }
};
const getNodeTypeName = (typescript, checker, node) => {
    const type = checker.getTypeAtLocation(node);
    return getTypeName(typescript, checker, type);
};
class TypeResolver {
    constructor(typescript, getTypeChecker) {
        this.typescript = typescript;
        this.getTypeChecker = getTypeChecker;
    }
    getType(node) {
        const checker = this.getTypeChecker();
        return getNodeTypeName(this.typescript, checker, node);
    }
}
exports.TypeResolver = TypeResolver;
//# sourceMappingURL=type-resolver.js.map